package com.gic.haoban.manage.web.controller.order;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.business.order.dto.ordermanage.ListOrderDTO;
import com.gic.business.order.dto.ordermanage.ListOrderItemDTO;
import com.gic.business.order.dto.ordermanage.OrderDetailDTO;
import com.gic.business.order.dto.ordermanage.OrderInfoDTO;
import com.gic.business.order.enums.OrderEnum;
import com.gic.business.order.qdto.ordermanage.ListOrderManageQDTO;
import com.gic.business.order.qdto.ordermanage.OrderItemShipmentQDTO;
import com.gic.business.order.qdto.ordermanage.OrderShipmentQDTO;
import com.gic.business.order.qdto.ordermanage.UpdateOrderLogisticsQDTO;
import com.gic.business.order.service.ordermanage.WebOrderManageApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.manage.web.controller.marketing.MemberMarketingController;
import com.gic.haoban.manage.web.vo.order.MemberInfoVO;
import com.gic.haoban.manage.web.vo.order.OrderCountVO;
import com.gic.haoban.manage.web.vo.order.OrderDetailItemVO;
import com.gic.haoban.manage.web.vo.order.OrderDetailVO;
import com.gic.haoban.manage.web.vo.order.OrderListItemVO;
import com.gic.haoban.manage.web.vo.order.OrderListVO;

@RestController
@RequestMapping("/order")
public class GicOrderController {

    private static final org.apache.logging.log4j.Logger logger = LogManager.getLogger(MemberMarketingController.class);

    @Autowired
    private WebOrderManageApiService webOrderManageApiService;

    // 发货订单列表 (待发货2+待收货3)
    @RequestMapping("list-order")
    public RestResponse<Object> orderList(String enterpriseId, String storeId, String clerkId, String search,
            @RequestParam(defaultValue = "1") int status, BasePageInfo pageInfo) {
        // 查询门店
        ListOrderManageQDTO qdto = new ListOrderManageQDTO();
        qdto.setOrderStep(status);
        qdto.setBusinessType(OrderEnum.BusinessTypeEnum.MICRO_MALL.getType());
        qdto.setSearchOrderParam(search);
        // 配送方式 -1：所有 0：门店自提 1：快递发货
        qdto.setDeliveryType(1);
        qdto.setEnterpriseId(enterpriseId);
        qdto.setPageNum(pageInfo.getPageNum());
        qdto.setPageSize(pageInfo.getPageSize());
        ServiceResponse<Page<ListOrderDTO>> ret = webOrderManageApiService.listOrder(qdto);
        logger.info("订单信息={}", JSON.toJSONString(ret, true));
        if (ret.isSuccess()) {
            Page<ListOrderDTO> page = ret.getResult();
            List<ListOrderDTO> orderList = page.getResult();
            List<OrderListVO> voList = new ArrayList<>();
            for (ListOrderDTO order : orderList) {
                OrderListVO vo = EntityUtil.changeEntityNew(OrderListVO.class, order);
                List<ListOrderItemDTO> itemList = order.getOrderItemDTOList();
                List<OrderListItemVO> voItemList = EntityUtil.changeEntityListNew(OrderListItemVO.class, itemList);
                vo.setOrderItemList(voItemList);
                voList.add(vo);
            }
            Page<OrderListVO> retPage = PageHelperUtils.changePageToCurrentPage(ret.getResult(), OrderListVO.class);
            retPage.setResult(voList);
            return RestResponse.successResult(retPage);
        }
        return RestResponse.failure(ret.getCode(), ret.getMessage());
    }

    // 数量查询(待发货+待收货)
    // orderType:2待发货 3待收货
    @RequestMapping("order-deliver-count")
    public RestResponse<Object> orderCount(String enterpriseId, String clerkId, String searchParams,
            @RequestParam(defaultValue = "1") int orderType) {
        OrderCountVO vo = new OrderCountVO();
        return RestResponse.successResult(vo);
    }

    // 订单详情 (客户信息+收货地址+商品详情+订单信息 + 订单状态)
    @RequestMapping("order-detail")
    public RestResponse<Object> orderDetail(String enterpriseId, String clerkId, String searchParams,
            @RequestParam(defaultValue = "1") int orderType, String orderId) {
        ServiceResponse<OrderDetailDTO> ret = webOrderManageApiService.getOrderDetail(enterpriseId, orderId);
        if (!ret.isSuccess()) {
            return RestResponse.failure(ret.getCode(), ret.getMessage());
        }
        logger.info("订单信息={}", JSON.toJSONString(ret, true));
        OrderInfoDTO orderInfo = ret.getResult().getOrderInfo();
        OrderDetailVO orderDetailVO = EntityUtil.changeEntityByJSON(OrderDetailVO.class,
                ret.getResult().getOrderInfo());
        List<OrderDetailItemVO> orderItemList = EntityUtil.changeEntityListNew(OrderDetailItemVO.class,
                ret.getResult().getOrderItemList());
        orderDetailVO.setOrderItemList(orderItemList);
        MemberInfoVO member = new MemberInfoVO();
        member.setMemberId("");
        member.setMemberImage("");
        member.setAddressPhone(orderInfo.getBuyerPhone());
        member.setAddressName(orderInfo.getBuyerName());
        member.setAddressDetail(orderInfo.getAddressInfo());
        orderDetailVO.setMember(member);
        
        return RestResponse.successResult(orderDetailVO);
    }

    // 订单发货商品列表
    @RequestMapping("order-deliver-list")
    public RestResponse<Object> orderDeliverList(String enterpriseId, String orderId) {
        ServiceResponse<OrderDetailDTO> resp = this.webOrderManageApiService.getOrderDetail(enterpriseId, orderId);
        List<ListOrderItemDTO> list = resp.getResult().getOrderItemList();
        List<OrderListItemVO> voList = new ArrayList<>();
        for (ListOrderItemDTO item : list) {
            if (StringUtils.isEmpty(item.getLogisticsId())) {
                OrderListItemVO vo = EntityUtil.changeEntityNew(OrderListItemVO.class, item);
                voList.add(vo);
            }
        }
        return RestResponse.successResult(voList);
    }

    // 发货
    @RequestMapping("order-logistics-deliver")
    public RestResponse<Object> orderDeliver(String enterpriseId, String orderId, String storeId,
            List<String> orderItemIdList, String companyCode, String logisticsNo, String companyId,
            String companyName) {
        OrderShipmentQDTO qdto = new OrderShipmentQDTO();
        qdto.setEnterpriseId(enterpriseId);
        qdto.setLogisticsCompanyCode(companyCode);
        qdto.setLogisticsCompanyName(companyName);
        qdto.setLogisticsCompanyId(companyId);
        qdto.setCourierNumber(logisticsNo);
        qdto.setOrderId(orderId);
        // 门店发货
        qdto.setDeliveryMode(2);
        // 好办
        qdto.setSource(2);
        List<OrderItemShipmentQDTO> list = new ArrayList<>();
        orderItemIdList.forEach(o -> {
            OrderItemShipmentQDTO item = new OrderItemShipmentQDTO();
            item.setOrderItemId(o);
            item.setStoreId(storeId);
            list.add(item);
        });
        qdto.setOrderItemShipment(list);
        ServiceResponse<Boolean> response = webOrderManageApiService.orderShipment(qdto);
        if (!response.isSuccess()) {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
        return RestResponse.successResult(response.getResult());
    }

    // 修改物流
    @RequestMapping("modify-order-logistics")
    public RestResponse<Object> modifyOrderDeliver(String enterpriseId, String orderId, String storeId,
            List<String> orderItemIdList, String companyCode, String logisticsNo, String companyId,
            String companyName) {
        UpdateOrderLogisticsQDTO qdto = new UpdateOrderLogisticsQDTO();
        qdto.setEnterpriseId(enterpriseId);
        qdto.setLogisticsCompanyCode(companyCode);
        qdto.setLogisticsCompanyName(companyName);
        qdto.setLogisticsCompanyId(companyId);
        qdto.setCourierNumber(logisticsNo);
        qdto.setOrderId(orderId);
        qdto.setOrderItemIds(orderItemIdList);
        this.webOrderManageApiService.updateOrderLogistics(qdto);
        return RestResponse.successResult();
    }
}
