package com.gic.haoban.manage.web.controller.order;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.business.order.api.service.order.AfterSalesManageApiService;
import com.gic.business.order.dto.ordermanage.OrderExchangeConsultDTO;
import com.gic.business.order.dto.ordermanage.OrderExchangeDetailDTO;
import com.gic.business.order.dto.ordermanage.OrderRefundConsultDTO;
import com.gic.business.order.dto.ordermanage.OrderRefundDetailDTO;
import com.gic.business.order.dto.ordermanage.OrderRefundExchangeListDTO;
import com.gic.business.order.dto.ordermanage.RefundAddressDTO;
import com.gic.business.order.qdto.aftersales.ExchangeAddressQDTO;
import com.gic.business.order.qdto.aftersales.ExchangeRefusedQDTO;
import com.gic.business.order.qdto.aftersales.ExchangeSellerAgreeApplyQDTO;
import com.gic.business.order.qdto.aftersales.RefundBaseQDTO;
import com.gic.business.order.qdto.aftersales.RefundRefusedQDTO;
import com.gic.business.order.qdto.aftersales.RefundSellerAgreeQDTO;
import com.gic.business.order.qdto.ordermanage.RefundExchangeQDTO;
import com.gic.business.order.service.ordermanage.MallSellerAddressApiService;
import com.gic.business.order.service.ordermanage.OrderRefundApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.manage.web.vo.order.refund.OrderExchangeConsultVO;
import com.gic.haoban.manage.web.vo.order.refund.OrderExchangeDetailVO;
import com.gic.haoban.manage.web.vo.order.refund.OrderRefundConsultVO;
import com.gic.haoban.manage.web.vo.order.refund.OrderRefundDetailVO;
import com.gic.haoban.manage.web.vo.order.refund.RefundAddressVO;
import com.gic.haoban.manage.web.vo.order.refund.SellerAddressVO;

@RestController
@RequestMapping("/refund")
public class GicOrderRefundController {

    @Autowired
    private OrderRefundApiService orderRefundApiService;
    @Autowired
    private MallSellerAddressApiService mallSellerAddressApiService;
    @Autowired
    private AfterSalesManageApiService afterSalesManageApiService;

    /**
     * 
    * @Title: exchangeConsult
    * @Description: 售后列表
    * @author xugh
    * @param orderExchangeId
    * @return
    * @throws
     */
    @RequestMapping(value = "order-exchange-refund-list")
    @ResponseBody
    public RestResponse<Object> exchangeRefundList(RefundExchangeQDTO qdto) {
        ServiceResponse<Page<OrderRefundExchangeListDTO>> resp = orderRefundApiService.refundExchangePage(qdto);
        return RestResponse.successResult(resp.getResult());
    }

    // 详情
    @RequestMapping(value = "order-refund-detail")
    @ResponseBody
    public RestResponse<Object> refundDetail(String orderRefundId) {
        ServiceResponse<OrderRefundDetailDTO> resp = this.orderRefundApiService.refundDetail(orderRefundId);
        OrderRefundDetailVO vo = EntityUtil.changeEntityNew(OrderRefundDetailVO.class, resp.getResult());
        return RestResponse.successResult(vo);
    }

    // 详情
    @RequestMapping(value = "order-exchange-detail")
    @ResponseBody
    public RestResponse<Object> exchangeDetail(String orderExchangeId) {
        ServiceResponse<OrderExchangeDetailDTO> resp = this.orderRefundApiService.exchangeDetail(orderExchangeId);
        OrderExchangeDetailVO vo = EntityUtil.changeEntityNew(OrderExchangeDetailVO.class, resp.getResult());
        return RestResponse.successResult(vo);
    }

    // 审批退款单
    // 拒绝 1 , 同意2
    // 拒绝收货 1 同意收货2
    // step 1 申请 2收货
    @RequestMapping(value = "audit-refund-order")
    @ResponseBody
    public RestResponse<Object> auditRefund(String enterpriseId, String orderRefundId, String memberId,
            @RequestParam(defaultValue = "1") int auditStatus, String clerkId, String remark,
            @RequestParam(defaultValue = "1") int step, RefundBaseQDTO baseQDTO, double applyRefundPrice) {
        // 拒绝
        if (auditStatus == 1) {
            RefundRefusedQDTO qdto = EntityUtil.changeEntityNew(RefundRefusedQDTO.class, baseQDTO);
            qdto.setOrderRefundId(orderRefundId);
            if (step == 1) {
                this.afterSalesManageApiService.refundSellerRefusedApply(enterpriseId, memberId, qdto);
            }
            if (step == 2) {
                this.afterSalesManageApiService.refundSellerRefusedGoods(enterpriseId, memberId, qdto);
            }
        }
        // 同意
        if (auditStatus == 2) {
            RefundSellerAgreeQDTO qdto = EntityUtil.changeEntityNew(RefundSellerAgreeQDTO.class, baseQDTO);
            qdto.setApplyRefundPrice(applyRefundPrice);
            qdto.setOrderRefundId(orderRefundId);
            if (step == 1) {
                this.afterSalesManageApiService.refundSellerConfirmApply(enterpriseId, memberId, qdto);
            }
            if (step == 2) {
                this.afterSalesManageApiService.refundSellerConfirmGoods(enterpriseId, memberId, qdto);
            }
        }
        return RestResponse.successResult();
    }

    // 审批换货单
    // 拒绝 1 , 同意2
    // 拒绝收货 1 同意收货2
    // step 1 申请 2收货
    @RequestMapping(value = "audit-exchange-order")
    @ResponseBody
    public RestResponse<Object> auditExchange(String enterpriseId, String orderExchangeId, String memberId,
            @RequestParam(defaultValue = "1") int auditStatus, String clerkId, String remark, RefundBaseQDTO baseQDTO,
            ExchangeAddressQDTO address, @RequestParam(defaultValue = "1") int step) {
        // 拒绝
        if (auditStatus == 1) {
            ExchangeRefusedQDTO qdto = EntityUtil.changeEntityNew(ExchangeRefusedQDTO.class, baseQDTO);
            qdto.setOrderExchangeId(orderExchangeId);
            if (step == 1) {
                this.afterSalesManageApiService.exchangeSellerRefused(enterpriseId, memberId, qdto);
            }
            if (step == 2) {
                this.afterSalesManageApiService.exchangeSellerRefusedGoods(enterpriseId, memberId, qdto);
            }
        }
        // 同意
        if (auditStatus == 2) {
            ExchangeSellerAgreeApplyQDTO qdto = EntityUtil.changeEntityNew(ExchangeSellerAgreeApplyQDTO.class,
                    baseQDTO);
            if (step == 1) {
                qdto.setExchangeAddress(address);
                this.afterSalesManageApiService.exchangeSellerConfirm(enterpriseId, memberId, qdto);
            }
            if (step == 2) {
                this.afterSalesManageApiService.exchangeSellerConfirmGoods(enterpriseId, memberId, qdto);
            }
        }
        return RestResponse.successResult();
    }

    // 换货单发货
    @RequestMapping(value = "exchange-order-deliver")
    @ResponseBody
    public RestResponse<Object> exchangeDeliver(String enterpriseId, String orderRefundId,
            @RequestParam(defaultValue = "1") int auditStatus, String clerkId, String remark) {
        return RestResponse.successResult();
    }

    // 协商记录-退款
    @RequestMapping(value = "order-refund-consult")
    @ResponseBody
    public RestResponse<Object> refundConsult(String orderRefundId) {
        ServiceResponse<List<OrderRefundConsultDTO>> resp = orderRefundApiService.listRefundConsult(orderRefundId);
        List<OrderRefundConsultVO> list = EntityUtil.changeEntityListNew(OrderRefundConsultVO.class, resp.getResult());
        return RestResponse.successResult(list);
    }

    // 协商记录-换货
    @RequestMapping(value = "order-exchange-consult")
    @ResponseBody
    public RestResponse<Object> exchangeConsult(String orderExchangeId) {
        ServiceResponse<List<OrderExchangeConsultDTO>> resp = orderRefundApiService
                .listExchangeConsult(orderExchangeId);
        List<OrderExchangeConsultVO> list = EntityUtil.changeEntityListNew(OrderExchangeConsultVO.class,
                resp.getResult());
        return RestResponse.successResult(list);
    }

    // 收货地址
    @RequestMapping("get-seller-address")
    @ResponseBody
    public RestResponse<Object> refundAddressList(String enterpriseId, String orderId, String orderItemId) {
        SellerAddressVO vo = new SellerAddressVO();
        ServiceResponse<List<RefundAddressDTO>> resp = this.mallSellerAddressApiService.listRefundAddress(enterpriseId,
                orderId, orderItemId);
        List<RefundAddressDTO> list = resp.getResult();
        List<RefundAddressVO> sellerAddressList = new ArrayList<>();
        list.forEach(dto -> {
            RefundAddressVO item = EntityUtil.changeEntityNew(RefundAddressVO.class, dto);
            if (dto.getAddressType() == 0) {
                vo.setOwnerStoreAddress(item);
            }
            if (dto.getAddressType() == 1) {
                vo.setDeliverStoreAddress(item);
            }
            if (dto.getAddressType() == 2) {
                sellerAddressList.add(item);
            }
        });
        vo.setSellerAddressList(sellerAddressList);
        return RestResponse.successResult(vo);
    }

}
