package com.gic.haoban.manage.web.controller.order;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.business.order.dto.ordermanage.DispatchGoodsInfoDTO;
import com.gic.business.order.dto.ordermanage.ListOrderDTO;
import com.gic.business.order.enums.OrderEnum;
import com.gic.business.order.qdto.ordermanage.ListOrderManageQDTO;
import com.gic.business.order.service.ordermanage.WebOrderManageApiService;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.app.aggregation.api.dto.evaluation.CreateRecordRequest;
import com.gic.haoban.app.aggregation.api.dto.evaluation.OrderProductRequest;
import com.gic.haoban.app.aggregation.api.dto.evaluation.OrderVerificationRecordDTO;
import com.gic.haoban.app.aggregation.api.service.evaluation.OrderVerificationRecordApiService;
import com.gic.haoban.base.api.common.ShowFrontMessageException;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.web.controller.marketing.MemberMarketingController;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.order.integral.ClerkQo;
import com.gic.haoban.manage.web.vo.order.OrderListVO;
import com.gic.haoban.manage.web.vo.order.integral.CheckVO;
import com.gic.haoban.manage.web.vo.order.integral.OrderCountVO;
import com.gic.haoban.manage.web.vo.order.integral.ProductSku;
import com.gic.haoban.manage.web.vo.order.integral.ProductVO;
import com.gic.haoban.manage.web.vo.order.integral.ScanVO;
import com.gic.integralmall.api.dto.IntegralMallExchangeLogDTO;
import com.gic.integralmall.api.enums.IntegralMallExchangeLogStatusEnums;
import com.gic.integralmall.api.service.IntegralmallService;
import com.gic.integralmall.api.service.result.DubboResult;

/**
 * 
* @ClassName: GicOrderVerificationController
* @Description: 订单核销
* @author xugh
* @date 2023年2月24日 上午11:15:22
*
 */
@RestController
@RequestMapping("/order")
public class GicOrderVerificationController {

    private static final org.apache.logging.log4j.Logger logger = LogManager.getLogger(MemberMarketingController.class);

    @Autowired
    private WebOrderManageApiService webOrderManageApiService;
    @Autowired
    private IntegralmallService integralmallService;
    @Autowired
    private OrderVerificationRecordApiService orderVerificationRecordApiService;
    @Autowired
    private ClerkService clerkService;

    /**
     * 
    * @Title: scan
    * @Description: 商城订单列表查询
    * @author xugh
    * @param enterpriseId
    * @param storeId
    * @param clerkId
    * @param verificationType  2待核销记录 1已核销
    * @return
    * @throws
     */
    @RequestMapping("/mall-verification-list")
    public RestResponse<Object> mallOrderList(String enterpriseId, String storeId, String clerkId, BasePageInfo page,
            @RequestParam("1") int verificationType, BasePageInfo pageInfo) {
        ListOrderManageQDTO qdto = new ListOrderManageQDTO();
        qdto.setOrderShow(1);
        qdto.setOrderStep(9);
        qdto.setBusinessType(OrderEnum.BusinessTypeEnum.MICRO_MALL.getType());
        qdto.setDeliveryType(-1);
        qdto.setEnterpriseId(enterpriseId);
        qdto.setPageSizeZero(false);
        qdto.setCount(false);
        qdto.setPageNum(pageInfo.getPageNum());
        qdto.setPageSize(pageInfo.getPageSize());
        ServiceResponse<Page<ListOrderDTO>> ret = webOrderManageApiService.listOrder(qdto);
        logger.info("订单信息={}", JSON.toJSONString(ret, true));
        if (ret.isSuccess()) {
            Page<OrderListVO> retPage = PageHelperUtils.changePageToCurrentPage(ret.getResult(), OrderListVO.class);
            return RestResponse.successResult(retPage);
        }
        return RestResponse.failure(ret.getCode(), ret.getMessage());
    }

    @RequestMapping("/mall-verification-detail")
    public RestResponse<Object> mallOrderDetail(String enterpriseId, String orderId) {
        ServiceResponse<DispatchGoodsInfoDTO> resp = this.webOrderManageApiService.getDispatchGoodsInfo(enterpriseId,
                orderId);
        logger.info("核销订单详情={}", JSON.toJSONString(resp));
        if (resp.isSuccess()) {
            return RestResponse.successResult(resp.getResult());
        }
        return RestResponse.failure("9999", "订单查询不到");
    }

    /**
     * 核销订单
     */
    @RequestMapping("/verification")
    public RestResponse<Object> verification(String enterpriseId, String clerkId, String writeOffCode) {
        if (StringUtils.isEmpty(writeOffCode)) {
            return RestResponse.failure("9999", "核销码为空");
        }
        ClerkDTO clerkDTO = clerkService.getClerkByClerkId(clerkId);
        if (null == clerkDTO) {
            logger.info("积分商城订单核销开始，核销码：{} ,核销人不存在 ", writeOffCode);
            throw new ShowFrontMessageException("核销人不存在，核销失败！");
        }
        // 微商城核销
        if (writeOffCode.startsWith("M")) {

        } else {
            ClerkQo clerkQo = new ClerkQo();
            clerkQo.setClerkCode(clerkDTO.getClerkCode());
            clerkQo.setClerkId(clerkDTO.getClerkId());
            clerkQo.setStoreId(clerkDTO.getStoreId());
            clerkQo.setClerkName(clerkDTO.getClerkName());
            clerkQo.setEnterpriseId(enterpriseId);
            logger.info("积分商城订单核销开始，核销码：{} ,核销人：{} ", writeOffCode, clerkQo.getClerkId());
            enterpriseId = clerkDTO.getEnterpriseId();
            DubboResult<Boolean> check = integralmallService.haobanVerificationExchangeLog(enterpriseId, writeOffCode,
                    clerkQo.getClerkCode(), clerkQo.getClerkName(), clerkQo.getStoreId());
            logger.info("核销结果：{} ", JSON.toJSONString(check));
            CheckVO checkVO = new CheckVO();
            if (check.getSuccess()) {
                // 积分商城核销成功-添加核销记录
                String recordId = createIntegralRecord(enterpriseId, 1, clerkQo, writeOffCode);
                checkVO.setId(recordId);
                checkVO.setSuccess(Boolean.TRUE);
                checkVO.setCode(HaoBanErrCode.ERR_1.getCode());
                return RestResponse.successResult(checkVO);
            }
            /*
             * 核销失败情况 10001 兑换记录不存在 10002 当前订单状态无法核销 10003 该订单已经核销 10004 订单已经取消
             * 10005 10006 超期核销
             */
            checkVO.setCode(check.getErrorCode());
            checkVO.setMessage(check.getErrMessage());
            checkVO.setSuccess(Boolean.FALSE);
            if (check.getErrorCode() == HaoBanErrCode.ERR_100010.getCode()) {
                // 订单已核销
                OrderVerificationRecordDTO record = orderVerificationRecordApiService
                        .getRecordByClerkIdAndWriteOffCode(clerkQo.getClerkId(), writeOffCode);
                if (Objects.nonNull(record)) {
                    checkVO.setSuccess(Boolean.TRUE);
                    checkVO.setId(record.getRecordId());
                    return RestResponse.successResult(checkVO);
                }
                // 超期核销
                checkVO.setCode(HaoBanErrCode.ERR_100014.getCode());
                checkVO.setMessage(HaoBanErrCode.ERR_100014.getMsg());
            }
            if (!checkVO.getSuccess()) {
                HaobanResponse response = new HaobanResponse();
                response.setMessage(checkVO.getMessage());
                response.setErrorCode(checkVO.getCode());
                response.setResult(checkVO.getId());
                response.setDetailError("");
                return RestResponse.successResult(checkVO);
            }
        }
        return null;
    }

    /**
     * 扫码二维码
     */
    @RequestMapping("/scan")
    public RestResponse<Object> scan(String enterpriseId, String writeOffCode) {
        if (StringUtils.isEmpty(writeOffCode)) {
            return RestResponse.failure("9999", "核销码为空");
        }
        // 微商城
        if (writeOffCode.startsWith("M")) {
            return RestResponse.successResult();
        } else {
            // 积分商城
            DubboResult<IntegralMallExchangeLogDTO> integralResult = integralmallService
                    .getIntegralMallExchangeLogByWriteOffCode(enterpriseId, writeOffCode);
            if (null != integralResult && null != integralResult.getData()) {
                ScanVO vo = integralMallScan(integralResult.getData());
                return RestResponse.successResult(vo);
            }
        }
        return RestResponse.failure("9999", "订单查询不到");
    }

    // 核销数量
    @RequestMapping("/verification-count")
    public RestResponse<Object> verificationCountNew(String enterpriseId, String storeId, String clerkId) {
        Page<IntegralMallExchangeLogDTO> page = new Page<IntegralMallExchangeLogDTO>();
        page.setPageSize(1);
        page.setCurrentPage(1);
        Map<String, Object> params = new HashMap<>(8);
        params.put("enterpriseId", enterpriseId);
        params.put("proType", 2);
        params.put("changeType", 4);
        // 订单状态
        params.put("orderStatus", 1);
        params.put("goodsPointStoreId", storeId);
        page.setParams(params);
        page = integralmallService.listOnlineByPage(page);
        int integralUndoneCount = page.getTotalCount();
        int integralDoneCount = this.orderVerificationRecordApiService.countByGicClerkId(clerkId);
        // 查询微商城
        int mallDoneCount = 0;
        int mallUndoneCount = 0;
        OrderCountVO vo = new OrderCountVO();
        vo.setIntegralDoneCount(integralDoneCount);
        vo.setIntegralUndoneCount(integralUndoneCount);
        vo.setMallDoneCount(mallDoneCount);
        vo.setMallUndoneCount(mallUndoneCount);
        vo.setTotalDoneCount(integralDoneCount + mallDoneCount);
        vo.setTotalUndownCount(integralUndoneCount + mallUndoneCount);
        return RestResponse.successResult(vo);
    }

    private String createIntegralRecord(String enterpriseId, int status, ClerkQo clerkDTO, String writeOffCode) {
        if (logger.isDebugEnabled()) {
            logger.debug("创建核销记录........");
        }
        DubboResult<IntegralMallExchangeLogDTO> integralResult = integralmallService
                .getIntegralMallExchangeLogByWriteOffCode(enterpriseId, writeOffCode);
        IntegralMallExchangeLogDTO data = integralResult.getData();
        CreateRecordRequest record = new CreateRecordRequest();
        record.setRecordId(data.getIntegralMallProExchangeId());
        record.setStatus(status);
        record.setEnterpriseId(data.getEnterpriseId());
        record.setExchangeId(data.getIntegralMallProExchangeId());
        record.setFinishTime(data.getFinishTime());
        record.setWriteOffCode(data.getWriteOffCode());
        // 提货人
        record.setConsignee(data.getGoodsPerson());
        record.setConsigneePhone(data.getGoodsPhone());
        record.setPoint(data.getGoodsPoint());
        record.setAddress(data.getGoodsAddress());
        record.setTakeTime(data.getGoodsTime());
        record.setFetchCode(writeOffCode);
        // 核销人
        record.setClerkId(clerkDTO.getClerkId());
        record.setClerkCode(clerkDTO.getClerkCode());
        record.setClerkName(clerkDTO.getClerkName());
        record.setHaobanClerkId(clerkDTO.getHaobanClerkId());
        record.setHaobanEnterpriseId(clerkDTO.getHaobanEnterpriseId());
        // 商品信息

        OrderProductRequest productDTO = new OrderProductRequest();
        productDTO.setProName(data.getProName());
        productDTO.setProImage(data.getPhotoUrl());
        productDTO.setProCode(data.getProCode());
        productDTO.setProNumber(data.getGoodsNum());

        productDTO.setProSku(
                StringUtils.isBlank(data.getSku()) ? "[]" : JSONArray.parseArray(data.getSku()).toJSONString());
        productDTO.setProPrice((data.getCashShouldPay() / data.getGoodsNum()));
        productDTO.setProIntegral((data.getIntegralShouldPay() / data.getGoodsNum()));

        List<OrderProductRequest> productDTOS = new ArrayList<>(2);
        productDTOS.add(productDTO);
        record.setProducts(productDTOS);
        // 订单信息
        record.setAllCostIntegral(data.getAllCostIntegral());
        record.setPayCost(data.getPayCost());
        record.setOrderNo(data.getDefinedCode());
        record.setPayNumber(data.getPayNumber());
        record.setPayTime(data.getPayTime());
        return orderVerificationRecordApiService.createRecord(record);
    }

    private ScanVO integralMallScan(IntegralMallExchangeLogDTO exchangeLogDTO) {
        ScanVO scanVO = new ScanVO();
        scanVO.setHasOrder(1);
        scanVO.setOrderNo(exchangeLogDTO.getDefinedCode());
        scanVO.setEnterpriseId(exchangeLogDTO.getEnterpriseId());
        scanVO.setExchangeId(exchangeLogDTO.getIntegralMallProExchangeId());
        scanVO.setAllCostIntegral(exchangeLogDTO.getAllCostIntegral());
        scanVO.setPayCost(exchangeLogDTO.getPayCost());
        scanVO.setPayNumber(exchangeLogDTO.getPayNumber());
        scanVO.setPayTime(exchangeLogDTO.getPayTime());
        scanVO.setFinishTime(exchangeLogDTO.getFinishTime());

        ProductVO productVO = new ProductVO();
        productVO.setProName(exchangeLogDTO.getProName());
        productVO.setProCode(exchangeLogDTO.getProCode());
        productVO.setProSku(StringUtils.isBlank(exchangeLogDTO.getSku()) ? new ArrayList<>()
                : JSONArray.parseArray(exchangeLogDTO.getSku(), ProductSku.class));
        productVO.setProNumber(exchangeLogDTO.getGoodsNum());
        productVO.setProImage(exchangeLogDTO.getPhotoUrl());
        productVO.setProPrice((exchangeLogDTO.getCashShouldPay() / exchangeLogDTO.getGoodsNum()));
        productVO.setProIntegral((exchangeLogDTO.getIntegralShouldPay() / exchangeLogDTO.getGoodsNum()));

        scanVO.pushProduct(productVO);

        scanVO.setWriteOffCode(exchangeLogDTO.getWriteOffCode());
        scanVO.setFetchCode(exchangeLogDTO.getWriteOffCode());

        scanVO.setConsignee(exchangeLogDTO.getGoodsPerson());
        scanVO.setConsigneePhone(exchangeLogDTO.getGoodsPhone());
        scanVO.setPoint(exchangeLogDTO.getGoodsPoint());
        scanVO.setAddress(exchangeLogDTO.getGoodsAddress());
        scanVO.setTakeTime(exchangeLogDTO.getGoodsTime());
        // 兑换状态 -2兑换取消(退货) -1兑换无效 0兑换取消(礼品未付款) 11兑换中(加钱购待支付) 3已发货(礼品) 1兑换功成
        // 2领取(卡券) 4成功
        // 其实只需要 1 2 4
        scanVO.setStatus(exchangeLogDTO.getStatus() == 4 ? 1 : 0);
        scanVO.setChecked(0);
        if (scanVO.getStatus() == 1) {
            scanVO.setRecordId(exchangeLogDTO.getIntegralMallProExchangeId());
            OrderVerificationRecordDTO info = orderVerificationRecordApiService.recordInfo(scanVO.getRecordId());
            if (null != info) {
                // 说明已经核销了
                scanVO.setCreateTime(info.getCreateTime());
                scanVO.setChecked(1);
            }
            if (exchangeLogDTO.getStatus() == IntegralMallExchangeLogStatusEnums.CONFIRM.getStatus()) {
                scanVO.setCreateTime(exchangeLogDTO.getUpdateTime());
                scanVO.setChecked(1);
            }
        }
        return scanVO;
    }
}
